// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { __awaiter } from "tslib";
import { LocalCryptographyUnsupportedError } from "./models";
import { localSupportedAlgorithms } from "./algorithms";
import { convertJWKtoPEM } from "./conversions";
export function runOperation(key, operationName, algorithmName, data, signature) {
    return __awaiter(this, void 0, void 0, function* () {
        const algorithm = localSupportedAlgorithms[algorithmName];
        if (!algorithm) {
            throw new LocalCryptographyUnsupportedError(`Unsupported algorithm ${algorithm}`);
        }
        algorithm.validate(key, operationName);
        const operation = algorithm.operations[operationName];
        if (!operation) {
            throw new LocalCryptographyUnsupportedError(`Operation ${operationName} is not supported for algorithm ${algorithm}`);
        }
        const keyPEM = convertJWKtoPEM(key);
        return operation(keyPEM, data, signature);
    });
}
//# sourceMappingURL=runOperation.js.map